# Textsplitter package

This package is meant for structure recognition in PDF documents. It reads
the text from the document using the [pdfminer.six](https://pypi.org/project/pdfminer.six/) library.
Then, it uses in in-house rule-based algorithm to identify chapter titles,
enumerations, etc.
<br />
<br />
The package also offers functionality to summarize the
document part-by-part. This means that a chapter is summarized by summarizing the
summaries of the various sections beneath it. Likewise, the document is summarized
by summarizing the summaries of the chapters in it, and so on. The summaries
are generated by calling the [ChatGPT api](https://platform.openai.com/docs/api-reference/introduction)
This means that an access token to a paid ChatGPT account is required to use this functionality.
<br />
<br />
Finally, the package can also process the splitted and summarized document into
a local html page using an in-house parser.

## Installation

Installation works like: pip install pdftextsplitter

## Getting started

After installing the package, simply run: <br />
from pdftextsplitter import textsplitter <br />
mysplitter = textsplitter() <br />
mysplitter.set_documentpath("/absolute/path/to/the/folder/of/your/document/") <br />
mysplitter.set_documentname("your_document_name") # no .pdf-extension! <br />
mysplitter.set_outputpath("/absolute/path/to/where/you/want/your/outputs/") <br />
mysplitter.standard_params() <br />
mysplitter.process() <br />
<br />
After running the process-command, it can take a long time (up to an hour) to process your document, depending on the document size.
Afterwards, you can enjoy the results by opeing the outputs in the specified output folder (have a look at the html-file),
or you can further process the results using your own code.

## Setting some properties

If you wish to configure some parameters yourself, specify any of the following values between the standard_params-command and the process-command: <br />
mysplitter.set_histogramsize(100)               # Specifies the granularity of the histoggrams used to perform calculations on fontsize and whitelines in the document. <br />
mysplitter.set_headerboundary(1000.0)           # Specifies the boundary on the page between the header and the body of the document. <br />
mysplitter.set_footerboundary(55.0)             # Specifies the boundary on the page between the footer and the body of the document. <br />
mysplitter.set_MaxSummaryLength(50)             # Guideline for how long a summary of a single textpart should be in words (outcomes are not exact, but they can be steered with this). <br />
mysplitter.set_summarization_threshold(50)      # If a textpart has fewer words then this amount, it will not be summarized, but copied. This is done to save costs. <br />
mysplitter.set_LanguageModel("gpt-3.5-turbo")   # Choice for the Large Language Model you would ChatGPT like to use. <br />
mysplitter.set_LanguageChoice("Default")        # Language you would like to receive your summaries in from ChatGPT. <br />
mysplitter.set_LanguageTemperature(0.1)         # Temperature of the ChatGPT responses. <br />
mysplitter.set_MaxCallRepeat(20)                # If the ChatGPT api returns an unusable response or an error, we attempt the call again, until this maximum. Higher value <br />
                                                # will give a more trusted output, but also potentially higher costs. <br />
mysplitter.set_UseDummySummary(True)            # If this is set to True, summaries will be created in-house by selecting the first n words from the text, no ChatGPT calls will be used. <br />
                                                # This is a great way to experiment with the package without burning any money. Set it to False for receiving usable summaries. <br />
<br />
To let the package use your own paid ChatGPT-account (only required when set_UseDummySummary is put to False), run: <br />
mysplitter.ChatGPT_Key = "my personal access token" <br />
between the standard_params-command and the process-command.

## Controlling the terminal output

You can control the amount of messages received in the termal, by running <br />
mysplitter.process(0) <br />
instead. Values of -1, 0, 1, 2, etc. can be used. The higher the value, the more output you receive. -1 makes the terminal completely quiet and also suppresses the output-files. <br />
NOTE: Choosing option -1 here will also change the html-visualisation from a complete page to fraction of a page. This is because option -1 is meant for when this package as a building block
for a webapplication. Django is the preferred framework to use here. In that case, you want html that you can send to a Django-template. This is not the same as a standalone html-page.
So -1 gives you a html-output you can use in a webapp while other choices give you a standalone html-page.

## Controlling individual actions of the package

You can replace the process-command by any (or a combination of) of the following commands: <br />
mysplitter.document_metadata()                  # This will extract meta-data from the PDF like author, creation date, etc. suing the PyPDF2 library. <br />
mysplitter.read_native_toc("pdfminer")          # If the document has a table of contents in its meta-data, this is extracted. Supported libraries are pymupdf and pdfminer. <br />
mysplitter.textgeneration("pdfminer")           # This will read the text from the PDF document. Supported libraries are: pypdf2 (limited support), pymupdf, pdfminer. <br />
mysplitter.calculate_footerboundaries(0)        # This will calculate the cut-offs between headers, footers and body text in the document -1, 0, 1, 2, etc.=terminal output. <br />
mysplitter.export("default")                    # This will write the extracted text to a .txt-file for your convencience. <br />
mysplitter.fontsizehist()                       # This creates hisograms about all the font sizes encountered in the document. <br />
mysplitter.findfontregions()                    # This utilizes those histograms to decide which font sizes are large or small. <br />
mysplitter.whitelinehist()                      # Same as before, but now for white lines between textlines in the document. <br />
mysplitter.findlineregions()                    # Same as before, but now for white lines between textlines in the document. <br />
mysplitter.passinfo()                           # This will pass the calculated information to the internal rules so the structure elements in the document can be indentified. <br />
mysplitter.breakdown()                          # This will split the text in the document into distinct chapters, sections, etc. <br />
mysplitter.shiftcontents()                      # This refines the outcome of breakdown in the case the PDF document quotes any personal letters. <br />
mysplitter.calculatetree()                      # This calculates which sections belong to which chapter and so on. <br />
num_errors = mysplitter.layered_summary(0)      # This will call ChatGPT to create summaries for each part of your document. -1,0,1,2,etc. can be entered to control termal output. <br />
mysplitter.exportdecisions()                    # This will write the decisions per textline made by breakdown to a .txt-file for future analysis. <br />
mysplitter.exportalineas("default")             # This will write the outcome of breakdown to a .txt-file for future analysis. <br />
mysplitter.alineas_to_html()                    # This will generate a standalone html-page with all output from the package (enter "django" for incomplete html). <br />
<br />
Note that the process-command is nothing more then the sequential execution of all commands specified above. It is possible to only execute some of the commands while skipping others that you do not need. However, many commands need the outcome of some of the previous commands. So not all orders and combinations of the above commands will result in workable code. At the very least, one should remember to always execute passinfo immediatly before breakdown.

## Access to the produced data

After running the process-command (or some decomposition of the commands), you can directly access the produced data by retrieving the class members. For a full discussion of all members, we like to refer to the source code (textsplitter/Textpart/textsplitter.py), but we will discuss the most important members here. <br />
<br />
At first, each associated set-function discussed above under 'setting parameters' comes with a get-function to retrieve the parameters you used. This can also be done without setting the parameters in advance, in which case you will obtain the default-values. <br />
<br />
Secondly, the most important class member is mysplitter.textalineas which is an array of textalinea-objects (textsplitter/Textpart/textalinea.py). Useful members are: <br />
mysplitter.textalineas[index].texttitle: str            # The title of this textpart (chapter, section, etc.) <br />
mysplitter.textalineas[index].textlevel: int            # How deep this part occurs in the document. 0=entire document, 1=chapter, 2=section, etc. <br />
mysplitter.textalineas[index].summary: str              # The summary generated by ChatGPT from this textpart. <br />
mysplitter.textalineas[index].textcontent: list[str]    # The original text of this textpart, as obtained from the PDF. <br />
mysplitter.textalineas[index].nativeID: int             # The order in which the textparts are identified in the original PDF. <br />
mysplitter.textalineas[index].parentID: int             # The nativeID of the parent of this textpart. For example, a section belongs to a chapter (we call this the parent). <br />
                                                        # Note: a summary is generated by adding the textcontent of this textpart to the summaries of all children of this textpart. <br />
<br />
Some other useful members are: <br />
mysplitter.native_TOC: list[Native_TOC_Element] # see textsplitter/TextPart/read_native_toc.py This holds the table of contents from the PDF obtained from the metadata (if present). <br />
mysplitter.doc_metadata_author: str             # The author of the PDF, as obtained from its metadata (if present). <br />
mysplitter.doc_metadata_creator: str            # Same, but now the creator-field. <br />
mysplitter.doc_metadata_producer: str           # Same, but now the creator-field. <br />
mysplitter.doc_metadata_title: str              # Same, but now the creator-field. <br />
mysplitter.doc_metadata_subject: str            # Same, but now the creator-field. <br />
mysplitter.html_visualization: str              # The full html-code of the output (as produced by alineas_to_html()). <br />
mysplitter.api_wrongcalls_duetomaxwhile: int    # The number of calls to ChatGPT that could not be corrected because it would cross the limit set by set_MaxCallRepeat() <br />
                                                # For a trustworthy output, this number should equal zero. <br />
mysplitter.api_totalprice: float                # Total price in dollars that processing this document by ChatGPT took. For this field to contain useful information, one should set <br />
                                                # the following fields prior to running process: mysplitter.Costs_price & mysplitter.Costs_tokenportion <br />
                                                # these fields should match the pricing of your LLM choice, see [OpenAI pricing](https://openai.com/pricing) <br />

## Database models

This package comes with a collection of Django database models that can be used for integrating the functionality of this package into a Django webapplication. The details are documented in textsplitter/Django/analyze/ where you find another readme and a class inheritance diagram. Here, we will outline the basics:

## Testing tools

the package also provides some testing tools out-of-the-box. See the README.md-file in textsplitter/Tests/Tools/ for more information on how to work with them.
